// This file is a part of the TTRM Program;
// copyright (C) 2001-2002, Alexandros Droseltis;
// it is distributed under the terms of the
// GNU General Public Licence v.2 (read the file LICENCE)
#include"numtable.h"
#include<iostream.h>
#include<string.h>

// mknumbers: generates the extended numerical matrices of a table
// subset.

NumTable::NumTable(Matrix *Forms){
    row_mult=Forms[0].get_itsSeira(0).getPlithos();
    result=QString();
    result_hex=QString();
    mknumbers(Forms, 0, row_mult, num_table);
    if(row_mult==12){
	mknumbers(Forms, 0, 6, hex[0]);
	mknumbers(Forms, 6, 6, hex[1]);
    }
}

void NumTable::find(const char *line, bool order){
    int found;
    set_set(line);
    set2num();
    if(order){
	R_check_s(&found);
	if(row_mult==12) R_check_hex(&found);
    }
    else{
	T_check_s(&found);
	if(row_mult==12) T_check_hex(&found);
    }
    columns();
}

void NumTable::mknumbers(
    Matrix *Forms, int start, int plithos,  int out[][12][48]){

    int i, j, r, accid, head;

    for(i=0; i<4; i++){// 4 forms
	for(j=0; j<12; j++){// 12 transpositions
	    // select a part in the row
	    for(r=start; r<start+2*plithos-1; r++){
		accid=Forms[i]
		    .get_itsSeira(j)
		    .getNotes((r-start)%plithos+start)
		    .getAccid();
		head=Forms[i]
		    .get_itsSeira(j)
		    .getNotes((r-start)%plithos+start)
		    .getHead();
		out[i][j][r-start]=chromatic_small(accid, head);
	    }
	}
    }
}

int NumTable::chromatic_small(int x, int y){
   return(chromatic_big(x, y)%12);
}

int NumTable::chromatic_big(int x, int y){
  if(y%7<3) return(2*(y%7)+x+12*(y/7));
  else return(2*(y%7)+x-1+12*(y/7));
}

void NumTable::set_set(const char *line){
  char line2[100];
  int i=0;
  char tokensep[]=" \t,\n", *note;
  strcpy(line2, line);
  note=strtok(line2, tokensep);
  while(note!=NULL){
    strcpy(sep_notes[i], note);
    i++;
    note=strtok(NULL, tokensep);
  }
  set_plithos=i;
}

void NumTable::set2num(void){
    int i, j;
    const char *notes[]={"his", "c", "deses",
			 "hisis", "cis", "des",
			 "cisis", "d", "eses",
			 "dis", "es", "feses",
			 "disis", "e", "fes",
			 "eis", "f", "geses",
			 "eisis", "fis", "ges",
			 "fisis", "g", "ases",
			 "gis", "as", "as",
			 "gisis", "a", "heses",
			 "ais", "b", "ceses",
			 "aisis", "h", "ces"};

    for(i=0; i<set_plithos; i++){
	for(j=0; j<36; j++){
	    if(strcmp(sep_notes[i], notes[j])==0){
		num_set[i]=j/3;
		break;
	    }
	}
    }
}

QString NumTable::get_result(){
    return result+result_hex;
}

void NumTable::R_check_s(int *found){
    int i, j, r, d, form;
    char *eidos[]={"P", "R", "I", "RI"};
    gefunden=0;
// Check the four matrices (P, R, I and RI):
    for(form=0; form<4; form++){
	// Check the 12 transpositions of the matrix:
	for(i=0; i<12; i++){
	    // d shows how many tones are indentified.
	    // At the beginning there are 0 tones indentified.
	    d=0;
	    /* Check every group of set_plithos tones of the transposition.
	       Hence the multitude if the group to search is set_plithos tones,
	       the width of the search-window must include all combinations
	       that are: 0<group<set_plithos ... 11<group<11+set_plithos. */
//	    for(j=0; j<11+set_plithos; j++){
	    for(j=0; j<row_mult; j++){
		/* If you indentify the first not of the group... */
		if(num_set[0]==num_table[form][i][j]){
		    /*...then d=1 (1 indentified tone) */
		    d=1;
		    /* Check the rest of the group (set_plithos-1 tones). */
		    for(r=0; r<set_plithos-1; r++){
			/* If the r+1 tone is indentical, d++
			   and keep on searching. */
			if(num_set[r+1]==num_table[form][i][j+r+1]) d++;
			/* If one tone is NOT indentical, stop searching. */ 
			else break;
		    }
		    /* If all tones were indentified, the value of d must
		       be equal the multitude of the given group,
		       i.e. d=set_plithos. If this happens... */
		    if(d==set_plithos){
			*found=1;
// 			 ...print the name of the row form;
// 			   that consists of:
// 			   a) the type of the row-form (P, R, I or RI),
// 			   b) the transposition indicator (i),
// 			   c) the beggining note of the group in the row-form.
			words[gefunden]=QString("%1(%2)\t%3")
			    .arg(eidos[form])
			    .arg(i)
			    .arg(j+1);
			gefunden++;
			//break;// Stop searching at that transposition
			      // and go to the next.

		    }
// 		    If all tones were NOT indentified, then
// 		    d!=set_plithos. That means that the given group is
// 		    not existent in this transposition at that point
// 		    The search must continue, if the row has
// 		    repetitive tones, and must break, if the row has
// 		    no repetitive tones.
		    //else break;
		}
	    }
	}
    }
}

void NumTable::R_check_hex(int *found){
    int i, j, r, d, form, type;
    char *eidos[]={"P", "R", "I", "RI"};
    char *c_type[]={"a", "b"};

    gefunden1=0;
    for(form=0; form<4; form++){
	for(i=0; i<12; i++){
	    for(type=0; type<2; type++){
		d=0;
//		for(j=0; j<5+set_plithos; j++){
		for(j=0; j<6; j++){
		    if(num_set[0]==hex[type][form][i][j]){
			d=1;
			for(r=0; r<set_plithos-1; r++){
			    if(num_set[r+1]==hex[type][form][i][j+r+1]) d++;
			    else break;
			}
			if(d==set_plithos){
			    *found=1;
			    if(j+set_plithos>6){
				words_hex[gefunden1]=QString("%1(%2)%3\t%4")
				    .arg(eidos[form])
				    .arg(i)
				    .arg(c_type[type])
				    .arg(j+1+type*6);
				gefunden1++;
			    }
			    //break;
			}
			//else break;
		    }
		}
	    }
	}
    }
}

void NumTable::T_check_s(int *found){
    int z, i, j, r, d, form, t, k;
    char *eidos[]={"P", "R", "I", "RI"};
    gefunden=0;
    List myList;

    for(form=0; form<4; form++){
	for(i=0; i<12; i++){
// Start examining all possible begins of the array:
	    for(t=0; t<row_mult; t++){
		myList.init(set_plithos, num_set);
// Search the set_plithos note beginning at t:
		for(k=0; k<set_plithos; k++){
		    r=myList.get_mult();
		    for(j=0; j<r; j++){
			if(myList.isEqual(j, num_table[form][i][t+k])){
			    myList.remove(j);
			    break;
			}
		    }
		    if(j==r) break;
		    else if (myList.get_mult()==0){
			*found=1;
			words[gefunden]=QString("%1(%2)\t%3")
			    .arg(eidos[form])
			    .arg(i)
			    .arg(t+k-set_plithos+2);
			gefunden++;
		    }
		}
	    }
	}
    }
}
void NumTable::T_check_hex(int *found){
    int z, i, j, r, d, form, type, t, k;
    char *eidos[]={"P", "R", "I", "RI"};
    char *c_type[]={"a", "b"};
    gefunden1=0;
    List myList;

    for(form=0; form<4; form++){
	for(i=0; i<12; i++){
	    for(type=0; type<2; type++){
		for(t=0; t<6; t++){
		    myList.init(set_plithos, num_set);
// Search the set_plithos note beginning at i:
		    for(k=0; k<set_plithos; k++){
			r=myList.get_mult();
			for(j=0; j<r; j++){
			    if(myList.isEqual(j, hex[type][form][i][t+k])){
				myList.remove(j);
				break;
			    }
			}
			if(j==r) break;
			else if(myList.get_mult()==0){
			    *found=1;
			    if(t+set_plithos>6){
				words_hex[gefunden1]=QString("%1(%2)%3\t%4")
				    .arg(eidos[form])
				    .arg(i)
				    .arg(c_type[type])
				    .arg(t+k-set_plithos+2+type*6);
				gefunden1++;
			    }
			}
		    }
		}
	    }
	    
	}
    }
}

void NumTable::columns(void){
    int i, j;
    for(i=0; i<12; i++){
	for(j=0; j<4; j++){
	    result+=words[i+j*12];
	    if(j<3) result+="\t";
	    else result+="\n";
	}
    }

    result+="--------------------\n";

    if(row_mult==12){
	for(i=0; i<12; i++){
	    for(j=0; j<8; j++){
		result_hex+=words_hex[i+j*12];
		if(j<7) result_hex+="\t";
		else result_hex+="\n";
	    }
	}
    }
}
